@@ -92,10 +92,11 @@ class simulator:
         if 'init' in simItem:
             self.init = simItem['init']
 
-        for handler in simItem['handlers']:
-            name = list(handler.keys())[0]
-            code  = list(handler.values())[0]
-            self.handlers[name] = code
+        if 'handlers' in simItem:
+            for handler in simItem['handlers']:
+                name = list(handler.keys())[0]
+                code  = list(handler.values())[0]
+                self.handlers[name] = code
 
 class fieldVal:
     def __init__(self, name):
@@ -511,6 +512,7 @@ def addStandardPackets(protocol):
     ack = packetDesc("Ack", protocol)
     icd = fieldDesc("icd", "uint32")
     icd.isRequired = True
+    icd.format = 'FORMAT_HEX'
     icd.setPrefix(protocol.prefix)
     icd.desc = "CRC Hash of protocol description. This is used to verify endpoints are using the same protocol"
     ping.desc = "This message requests an Ack from a remote device to test connectivity"
@@ -522,6 +524,7 @@ def addStandardPackets(protocol):
     ack.standard = True
     protocol.addField(icd)
     ping.addField(icd)
+    ack.addField(icd)
     protocol.addPacket(ping)
     protocol.addPacket(ack)
 
